﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("MCVTripleMove", "A move on a multi component vector that is specified by a rotation-value, two indexes and a groupNr.")]
  [StorableClass]
  public class MCVTripleMove : MultiComponentVectorMove {   
    [Storable]
    public int OldGroup { get; protected set; }
    [Storable]
    public int NewGroup { get; protected set; }
    [Storable]
    public int Index { get; protected set; }    
    [Storable]
    public int TargetIndex { get; protected set; }
    [Storable]
    public bool Rotation { get; protected set; }

    [StorableConstructor]
    protected MCVTripleMove(bool deserializing) : base(deserializing) { }
    protected MCVTripleMove(MCVTripleMove original, Cloner cloner)
      : base(original, cloner) {     
      this.OldGroup = original.OldGroup;
      this.NewGroup = original.NewGroup;
      this.Index = original.Index;       
      this.TargetIndex = original.TargetIndex;
      this.Rotation = original.Rotation;
    }
    public MCVTripleMove(int oldGroup, int newGroup, int index, int targetIndex, bool rotation, MultiComponentVectorEncoding multiComponentVector)
      : base(multiComponentVector) { 
        OldGroup = oldGroup;
        NewGroup = newGroup;
        Index = index;        
        TargetIndex = targetIndex;
        Rotation = rotation;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MCVTripleMove(this, cloner);
    }

    public override IPackingSolutionEncoding GetSolutionAfterMove() {
      var result = MultiComponentVector.Clone(new Cloner()) as MultiComponentVectorEncoding;
      if (Rotation)
        result = SingleItemRotationMove.GetVectorAfterMove(result, OldGroup, Index);
      if (!OldGroup.Equals(NewGroup)) {
        result = SingleGroupingMove.GetVectorAfterMove(result, OldGroup, Index, NewGroup);
        int newIndex = result.PackingInformations[NewGroup].Count - 1;
        if (!newIndex.Equals(TargetIndex))
          result = ChangePositionMove.GetVectorAfterMove(result, NewGroup, newIndex, TargetIndex);
      } else if (!Index.Equals(TargetIndex))
          result = ChangePositionMove.GetVectorAfterMove(result, NewGroup, Index, TargetIndex);
      return result;
    }

    public override System.Type GetMoveAttributeType() {
      return typeof(MCVTripleMoveAttribute);
    }

    public override string ToString() {
      return "TM(og=" + OldGroup + ",ng=" + NewGroup + ",i=" + Index + ",ti=" + TargetIndex + ",r=" + Rotation + ")";
    }


    public override MultiComponentVectorMoveAttribute GetAttribute(double quality) {
      return new MCVTripleMoveAttribute(Index, TargetIndex, MultiComponentVector.PackingInformations[OldGroup][Index].ItemID,
        Rotation, OldGroup, NewGroup, quality);
    }

    public override bool BelongsToAttribute(MultiComponentVectorMoveAttribute attribute, bool hardCriterion) {
      MCVTripleMoveAttribute actualAttribute = attribute as MCVTripleMoveAttribute;
      if (actualAttribute != null) {
        if (hardCriterion) {
          if (Index == actualAttribute.Index || TargetIndex == actualAttribute.TargetIndex
                  || MultiComponentVector.PackingInformations[OldGroup][Index].ItemID == actualAttribute.ItemID
                  //|| (OldGroup == actualAttribute.GroupBefore && NewGroup == actualAttribute.GroupAfter)
            )
            return true;
        } else {
          if ((Index == actualAttribute.Index || TargetIndex == actualAttribute.TargetIndex)
                  && MultiComponentVector.PackingInformations[OldGroup][Index].ItemID == actualAttribute.ItemID
                  //&& Rotation == actualAttribute.Rotation
                  //&& (OldGroup == actualAttribute.GroupBefore && NewGroup == actualAttribute.GroupAfter)
            )
            return true;
        }
      }
      return false;
    }
  }
}